﻿using fgxavi.Common.Business.Repositories;
using fgxavi.Common.Data;
using fgxavi.Common.Helpers;
using fgxavi.PagedQuery;
using Microsoft.Practices.Unity;
using System;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using System.Threading.Tasks;

namespace fgxavi.Common.Business.Engines
{
    public class EngineImpl<TEntity> : IEngine<TEntity>
        where TEntity : class
    {
        public Task<int> CountAsync
        {
            get
            {
                var container = Singleton<IUnityContainer, UnityContainer>.Instance;
                var repo = container.Resolve<IQueryableRepository<TEntity>>();
                var result = repo.CountAsync;
                return result;
            }
        }

        public Task<int> CountByAsync(Func<TEntity, bool> predicate)
        {
            var container = Singleton<IUnityContainer, UnityContainer>.Instance;
            var repo = container.Resolve<IQueryableRepository<TEntity>>();
            var result = Task<int>.Run(() => repo.Find().Count(predicate));
            return result;
        }

        public Task<int> CountByAsync(Expression<Func<TEntity, bool>> predicate)
        {
            var container = Singleton<IUnityContainer, UnityContainer>.Instance;
            var repo = container.Resolve<IQueryableRepository<TEntity>>();
            var result = Task<int>.Run(() => repo.Find().Count(predicate));
            return result;
        }

        public Task<bool> ContainsAsync(TEntity item)
        {
            var container = Singleton<IUnityContainer, UnityContainer>.Instance;
            var repo = container.Resolve<IQueryableRepository<TEntity>>();
            var result = repo.ContainsAsync(item);
            return result;
        }

        public Task<bool> ContainsAsync(Func<TEntity, bool> predicate)
        {
            var container = Singleton<IUnityContainer, UnityContainer>.Instance;
            var repo = container.Resolve<IQueryableRepository<TEntity>>();
            var result = Task<bool>.Run(() => repo.Find().Any(predicate));
            return result;
        }

        public Task<bool> ContainsAsync(Expression<Func<TEntity, bool>> predicate)
        {
            var container = Singleton<IUnityContainer, UnityContainer>.Instance;
            var repo = container.Resolve<IQueryableRepository<TEntity>>();
            var result = Task<bool>.Run(() => repo.Find().Any(predicate));
            return result;
        }

        public Task<int> InsertAsync(TEntity item)
        {
            var container = Singleton<IUnityContainer, UnityContainer>.Instance;
            var repo = container.Resolve<IQueryableRepository<TEntity>>();
            var result = repo.InsertAsync(item);
            return result;
        }

        public Task<int> UpdateAsync(TEntity item)
        {
            var container = Singleton<IUnityContainer, UnityContainer>.Instance;
            var repo = container.Resolve<IQueryableRepository<TEntity>>();
            var result = repo.UpdateAsync(item);
            return result;
        }

        public Task<int> DeleteAsync(TEntity item)
        {
            var container = Singleton<IUnityContainer, UnityContainer>.Instance;
            var repo = container.Resolve<IQueryableRepository<TEntity>>();
            var result = repo.DeleteAsync(item);
            return result;
        }

        public Task<IPagedQuery<TEntity>> FindAsync(string path = null, int pageNumber = 1, int pageSize = 10)
        {
            var container = Singleton<IUnityContainer, UnityContainer>.Instance;
            var repo = container.Resolve<IQueryableRepository<TEntity>>();
            var result = repo.Find(path).ToPagedQueryAsync(pageNumber, pageSize);
            return result;
        }

        public Task<IPagedQuery<TEntity>> FindAsync(System.Func<TEntity, bool> predicate, string path = null, int pageNumber = 1, int pageSize = 10)
        {
            var container = Singleton<IUnityContainer, UnityContainer>.Instance;
            var repo = container.Resolve<IQueryableRepository<TEntity>>();
            var result = repo.Find(predicate, path).ToPagedQueryAsync(pageNumber, pageSize);
            return result;
        }

        public Task<IPagedQuery<TEntity>> FindAsync(Expression<Func<TEntity, bool>> predicate, string path = null, int pageNumber = 1, int pageSize = 10)
        {
            var container = Singleton<IUnityContainer, UnityContainer>.Instance;
            var repo = container.Resolve<IQueryableRepository<TEntity>>();
            var result = repo.Find(predicate, path).ToPagedQueryAsync(pageNumber, pageSize);
            return result;
        }
    }
}
