﻿using fgxavi.PagedQuery;
using System;
using System.Data.Entity;
using System.Linq.Expressions;
using System.Threading.Tasks;

namespace fgxavi.Common.Business.Engines
{
    public interface IEngine<TEntity>
        where TEntity : class
    {
        Task<int> CountAsync { get; }
        Task<int> CountByAsync(Func<TEntity, bool> predicate);
        Task<int> CountByAsync(Expression<Func<TEntity, bool>> predicate);
        Task<bool> ContainsAsync(TEntity item);
        Task<bool> ContainsAsync(Func<TEntity, bool> predicate);
        Task<bool> ContainsAsync(Expression<Func<TEntity, bool>> predicate);
        Task<int> InsertAsync(TEntity item);
        Task<int> UpdateAsync(TEntity item);
        Task<int> DeleteAsync(TEntity item);
        Task<IPagedQuery<TEntity>> FindAsync(string path = null, int pageNumber = 1, int pageSize = 10);
        Task<IPagedQuery<TEntity>> FindAsync(Func<TEntity, bool> predicate, string path = null, int pageNumber = 1, int pageSize = 10);
        Task<IPagedQuery<TEntity>> FindAsync(Expression<Func<TEntity, bool>> predicate, string path = null, int pageNumber = 1, int pageSize = 10);
    }
}
