﻿using fgxavi.Common.Helpers;
using fgxavi.PagedQuery;
using Microsoft.Practices.Unity;
using System.Linq;

namespace fgxavi.Common.Data.Translators
{
    public static class TranslatorExtensions
    {
        public static TResult Translate<T, TResult>(this T source)
            where T : class
            where TResult : class
        {
            var container = Singleton<IUnityContainer, UnityContainer>.Instance;
            var translator = container.Resolve<ITranslator>();
            var result = translator.Translate<T, TResult>(source);
            return result;
        }
        public static IQueryable<TResult> Translate<T, TResult>(this IQueryable<T> source)
            where T : class
            where TResult : class
        {
            var container = Singleton<IUnityContainer, UnityContainer>.Instance;
            var translator = container.Resolve<ITranslator>();
            var result = source.Select(s => translator.Translate<T, TResult>(s));
            return result;
        }
        public static PagedQuery<TResult> Translate<T, TResult>(this IPagedQuery<T> source)
            where T : class
            where TResult : class
        {
            var container = Singleton<IUnityContainer, UnityContainer>.Instance;
            var translator = container.Resolve<ITranslator>();
            var transform = source.Select(s => translator.Translate<T, TResult>(s));
            var result = new PagedQuery<TResult>(transform, source.PageNumber, source.PageSize);
            return result;
        }
    }
}
