﻿using System;
using System.Security.Cryptography;
using System.Text;

namespace fgxavi.Common.Helpers
{
    /// <summary>
    /// Classe generadora de claus hash de tipus SHA de 512 bits.
    /// </summary>
    public class HashHelper
    {

        #region Constants

        /// <summary>
        /// Longitud de la paraula Salt generada.
        /// </summary>
        public const int SaltByteSize = 4;

        #endregion

        #region Static method CreateRandomSalt

        /// <summary>
        /// Genera una paraula Salt aleatòria.
        /// </summary>
        /// <returns>Paraula Salt aleatòria.</returns>
        public static string CreateRandomSalt()
        {
            Byte[] saltBytes = new Byte[SaltByteSize];
            RNGCryptoServiceProvider rng = new RNGCryptoServiceProvider();
            rng.GetBytes(saltBytes);
            return Convert.ToBase64String(saltBytes);
        }

        #endregion

        #region Static method ComputeSaltedHash

        /// <summary>
        /// Genera la clau hash de tipus SHA de 512 bits d'una contrasenya
        /// mitjançant una paraula Salt.
        /// </summary>
        /// <param name="lambdaPassword">
        /// Funció lambda que retorna la contrasenya no xifrada en texte pla.
        /// </param>
        /// <param name="salt">
        /// Paraula Salt que s'utilitzarà per xifrar la contrasenya.
        /// </param>
        /// <returns>
        /// Clau hash basada en l'algoritme SHA512.
        /// </returns>
        /// <remarks>
        /// S'utilitza la funció lambda com a paràmetre de la contrasenya perquè 
        /// la contrasenya sense xifrar no viatgi en texte pla 
        /// restant vulnerabilitat a possibles intrusions del codi de l'aplicació.
        /// </remarks>
        public static string ComputeSaltedHash(Func<string> lambdaPassword, string salt)
        {
            // Crea un array de bytes de la contrasenya:
            UnicodeEncoding encoder = new UnicodeEncoding();
            Byte[] secretBytes = encoder.GetBytes(lambdaPassword());

            // Crea un array de bytes de la paraula Salt
            Byte[] saltBytes = Convert.FromBase64String(salt);

            // Uneix ambdos arrays per xifrar la contrasenya conjuntament amb la paraula Salt:
            Byte[] toHash = new Byte[secretBytes.Length + saltBytes.Length];
            Array.Copy(secretBytes, 0, toHash, 0, secretBytes.Length);
            Array.Copy(saltBytes, 0, toHash, secretBytes.Length, saltBytes.Length);

            // Calcula el hash mitjançant l'algoritme SHA512:
            SHA512 sha512 = SHA512.Create();
            Byte[] computedHash = sha512.ComputeHash(toHash);

            // Retorna el hash en cadena de texte:
            return Convert.ToBase64String(computedHash);
        }

        #endregion

    }
}
