﻿using System;

namespace fgxavi.Common.Helpers
{
    public class Singleton<T>
        where T : class, new()
    {
        /// <summary>
        /// Variable estàtica per a la instància.
        /// </summary>
        /// <remarks>Es necessita utilitzar una funció lambda ja que el constructor és privat.</remarks>
        private static readonly Lazy<T> instance = new Lazy<T>(() => new T());

        /// <summary>
        /// Constructor protegit per evitar la instanciació directa.
        /// </summary>
        protected Singleton()
        {
        }

        /// <summary>
        /// Propietat per accedir a la instància.
        /// </summary>
        public static T Instance { get { return instance.Value; } }
    }
    public class Singleton<TInterface, TClass>
        where TInterface : class
        where TClass : class, TInterface, new()
    {
        /// <summary>
        /// Variable estàtica per a la instància.
        /// </summary>
        /// <remarks>Es necessita utilitzar una funció lambda ja que el constructor és privat.</remarks>
        private static readonly Lazy<TInterface> instance = new Lazy<TInterface>(() => new TClass());

        /// <summary>
        /// Constructor protegit per evitar la instanciació directa.
        /// </summary>
        protected Singleton()
        {
        }

        /// <summary>
        /// Propietat per accedir a la instància.
        /// </summary>
        public static TInterface Instance { get { return instance.Value; } }
    }
}
