﻿using Microsoft.Data.Edm;
using Microsoft.Data.Edm.Library;
using System.Linq;
using System.Net.Http;
using System.Web.Http.Controllers;
using System.Web.Http.OData.Routing;
using System.Web.Http.OData.Routing.Conventions;

namespace fgxavi.Common.Services
{
    public class CountODataRoutingConvention : EntitySetRoutingConvention
    {
        public override string SelectAction(ODataPath odataPath, HttpControllerContext controllerContext, ILookup<string, HttpActionDescriptor> actionMap)
        {
            string[] actions;
            if (controllerContext.Request.Method == HttpMethod.Get)
                switch (odataPath.PathTemplate)
                {
                    case "~/entityset":
                        actions = new string[] { "GetAsync", "Get" };
                        foreach (var action in actions)
                            if (actionMap.Contains(action))
                                return action;
                        break;
                    case "~/entityset/$count":
                        actions = new string[] { "GetCountAsync", "GetCount" };
                        foreach (var action in actions)
                            if (actionMap.Contains(action))
                                return action;
                        break;
                    default:
                        break;
                }
            return null;
        }
    }

    public class CountODataPathHandler : DefaultODataPathHandler
    {
        protected override ODataPathSegment ParseAtEntityCollection(IEdmModel model, ODataPathSegment previous, IEdmType previousEdmType, string segment)
        {
            if (segment == "$count")
            {
                return new CountPathSegment();
            }
            return base.ParseAtEntityCollection(model, previous, previousEdmType, segment);
        }
    }

    public class CountPathSegment : ODataPathSegment
    {
        public override string SegmentKind
        {
            get
            {
                return "$count";
            }
        }

        public override IEdmType GetEdmType(IEdmType previousEdmType)
        {
            return EdmCoreModel.Instance.FindDeclaredType("Edm.Int32");
        }

        public override IEdmEntitySet GetEntitySet(IEdmEntitySet previousEntitySet)
        {
            return previousEntitySet;
        }

        public override string ToString()
        {
            return "$count";
        }
    }
}
