﻿using fgxavi.Common.Business.Engines;
using fgxavi.Common.Data.Translators;
using fgxavi.Common.Helpers;
using fgxavi.PagedQuery;
using Microsoft.Practices.Unity;
using System;
using System.Data.Entity;
using System.Linq.Expressions;
using System.Threading.Tasks;

namespace fgxavi.Common.Services
{
    public class ServiceDirectImpl<TEntity, TContract> : IService<TEntity, TContract>
        where TEntity : class
        where TContract : class
    {
        protected IEngine<TEntity> engine;
        public ServiceDirectImpl()
        {
            var container = Singleton<IUnityContainer, UnityContainer>.Instance;
            engine = container.Resolve<IEngine<TEntity>>();
        }
        public ServiceDirectImpl(IEngine<TEntity> engine)
        {
            this.engine = engine;
        }

        public Task<int> CountAsync()
        {
            var result = engine.CountAsync;
            return result;
        }

        //public Task<int> CountByAsync(Func<TEntity, bool> predicate)
        //{
        //    var result = engine.CountByAsync(predicate);
        //    return result;
        //}

        //public Task<int> CountByExpAsync(Expression<Func<TEntity, bool>> predicate)
        //{
        //    var result = engine.CountByAsync(predicate);
        //    return result;
        //}

        public Task<bool> ContainsAsync(TContract item)
        {
            var entity = item.Translate<TContract, TEntity>();
            var result = engine.ContainsAsync(entity);
            return result;
        }

        //public Task<bool> ContainsByAsync(Func<TEntity, bool> predicate)
        //{
        //    var result = engine.ContainsAsync(predicate);
        //    return result;
        //}

        //public Task<bool> ContainsByExpAsync(Expression<Func<TEntity, bool>> predicate)
        //{
        //    var result = engine.ContainsAsync(predicate);
        //    return result;
        //}

        public Task<int> InsertAsync(TContract item)
        {
            var entity = item.Translate<TContract, TEntity>();
            var result = engine.InsertAsync(entity);
            return result;
        }

        public Task<int> UpdateAsync(TContract item)
        {
            var entity = item.Translate<TContract, TEntity>();
            var result = engine.UpdateAsync(entity);
            return result;
        }

        public Task<int> DeleteAsync(TContract item)
        {
            var entity = item.Translate<TContract, TEntity>();
            var result = engine.DeleteAsync(entity);
            return result;
        }

        public async Task<PagedQuery<TContract>> FindAsync(string path = null, int pageNumber = 1, int pageSize = 10)
        {
            var result = engine.FindAsync(path, pageNumber, pageSize);
            var translate = (await result).Translate<TEntity, TContract>();
            return translate;
        }

        //public async Task<PagedQuery<TContract>> FindByAsync(Func<TEntity, bool> predicate, string path = null, int pageNumber = 1, int pageSize = 10)
        //{
        //    var result = engine.FindAsync(predicate, path, pageNumber, pageSize);
        //    var translate = (await result).Translate<TEntity, TContract>();
        //    return translate;
        //}

        //public async Task<PagedQuery<TContract>> FindByExpAsync(Expression<Func<TEntity, bool>> predicate, string path = null, int pageNumber = 1, int pageSize = 10)
        //{
        //    var result = engine.FindAsync(predicate, path, pageNumber, pageSize);
        //    var translate = (await result).Translate<TEntity, TContract>();
        //    return translate;
        //}
    }
}
