﻿
IF OBJECT_ID(N'[dbo].[sp_Auditoria_CrearTriggers]', 'P') IS NOT NULL
	DROP PROCEDURE [dbo].[sp_Auditoria_CrearTriggers]
GO

CREATE PROCEDURE [dbo].[sp_Auditoria_CrearTriggers]
AS

IF NOT EXISTS(SELECT * FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME='Auditories')
BEGIN

	-- Creating table 'Auditories'
	CREATE TABLE [dbo].[Auditories] (
		[AuditoriaId] int IDENTITY(1,1) NOT NULL,
		[Tipus] char(1) NOT NULL,
		[Aplicacio] varchar(10) NOT NULL,
		[ProcId] binary(4) NOT NULL,
		[ProcName] varchar(128) NOT NULL,
		[Taula] varchar(128) NOT NULL,
		[PKCamps] varchar(max) NOT NULL,
		[PKValors] varchar(max) NOT NULL,
		[Camps] nvarchar(max) NOT NULL,
		[Valors] nvarchar(max) NOT NULL,
		[Moment] datetime DEFAULT (getdate()) NOT NULL,
		[Usuari] varchar(10) NOT NULL,
		[Ip] binary(16) NULL,
		[IpAdress] varchar(39) NULL,
		[UsuariHost] varchar(20) NULL,
		[Hostname] varchar(63) NULL
	);

	-- Creating primary key on [AuditoriaId] in table 'Auditories'
	ALTER TABLE [dbo].[Auditories]
	ADD CONSTRAINT [PK_Auditories]
		PRIMARY KEY CLUSTERED ([AuditoriaId] ASC);

	-- Creating table 'Auditories_Updated'
	CREATE TABLE [dbo].[Auditories_Updated] (
		[ValorsAnt] nvarchar(max)  NOT NULL,
		[CampsMod] nvarchar(max)  NOT NULL,
		[ValorsMod] nvarchar(max)  NOT NULL,
		[ValorsAntMod] nvarchar(max)  NOT NULL,
		[AuditoriaId] int  NOT NULL
	);

	-- Creating primary key on [AuditoriaId] in table 'Auditories_Updated'
	ALTER TABLE [dbo].[Auditories_Updated]
	ADD CONSTRAINT [PK_Auditories_Updated]
		PRIMARY KEY CLUSTERED ([AuditoriaId] ASC);

	-- Creating foreign key on [AuditoriaId] in table 'Auditories_Updated'
	ALTER TABLE [dbo].[Auditories_Updated]
	ADD CONSTRAINT [FK_Updated_inherits_Auditoria]
		FOREIGN KEY ([AuditoriaId])
		REFERENCES [dbo].[Auditories]
			([AuditoriaId])
		ON DELETE CASCADE ON UPDATE NO ACTION;

END

SET NOCOUNT ON

-- Defineix el separador dels camps:
DECLARE
@separador char(1)
SELECT @separador='#'

-- Declara les variables del script de la creació del trigger de cada taula:
DECLARE @sql varchar(max), @Taula sysname

-- Declara i obté el cursor amb totes les taules on s'afegiran els disparadors:
DECLARE table_cursor CURSOR FOR
SELECT TABLE_NAME
FROM INFORMATION_SCHEMA.Tables
WHERE TABLE_TYPE='BASE TABLE' AND TABLE_NAME!='sysdiagrams' AND NOT TABLE_NAME LIKE 'Auditories%'
ORDER BY TABLE_NAME

-- Obre el cursor de les taules i obté la primera taula:
OPEN table_cursor
FETCH NEXT FROM table_cursor 
INTO @Taula

-- Itera les taules del cursor mentre n'hi hagi:
WHILE @@FETCH_STATUS = 0
BEGIN

EXEC('IF OBJECT_ID (''tr_' + @Taula + '_Auditoria'', ''TR'') IS NOT NULL DROP TRIGGER tr_' + @Taula + '_Auditoria')

-- #############################################
-- Genera el text del script que crearà el trigger de la taula:
SELECT @sql=
'CREATE TRIGGER tr_' + @Taula + '_Auditoria on ' + @Taula + '
FOR INSERT, UPDATE, DELETE AS
DECLARE
@sql varchar(max),
@PosCamp int, @NCamp varchar(64),
@bit int, @char int,
@Tipus char(1),
@PKWhere varchar(max), @SelPKCamps varchar(max), @SelPKValors varchar(max),
@SelNCamps varchar(max), @SelValAnts varchar(max), @SelValors varchar(max),
@SelNCampsMod varchar(max), @SelValAntsMod varchar(max), @SelValorsMod varchar(max),
@Moment varchar(23), @Usuari varchar(128)

' + -- Obté la llista de columnes modificades:
'
SELECT * INTO #ins FROM inserted
SELECT * INTO #del FROM deleted

' + -- Obté les columnes i SelValors de la clau primària del registre modificat a la taula:
'
SELECT
	@PKWhere=coalesce(@PKWhere + '' and'', '' on'') + '' i.'' + c.COLUMN_NAME + ''=d.'' + c.COLUMN_NAME,
	@SelPKCamps=coalesce(@SelPKCamps + '' + ''''' + @separador + ''''' + '','''') + '''''''' + c.COLUMN_NAME + '''''''',
	@SelPKValors=coalesce(@SelPKValors + '' + ''''' + @separador + ''''' + '','''') + ''convert(varchar(max), coalesce(i.'' + c.COLUMN_NAME + '',d.'' + c.COLUMN_NAME + ''))''
FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS pk,
	INFORMATION_SCHEMA.KEY_COLUMN_USAGE c
WHERE pk.TABLE_NAME=''' + @Taula + '''
	and pk.CONSTRAINT_TYPE=''PRIMARY KEY''
	and c.TABLE_NAME=pk.TABLE_NAME
	and c.CONSTRAINT_NAME=pk.CONSTRAINT_NAME

IF @PKWhere is null
BEGIN
	raiserror(''no PK on table %s'', 16, -1, ''' + @Taula + ''')
	return
END

' + -- S''obtenen les col·leccions de tots els camps del registre:
'
SELECT
	@SelNCamps=coalesce(@SelNCamps + '' + ''''' + @separador + ''''' + '','''', '''') + '''''''' + c.COLUMN_NAME + '''''''',
	@SelValAnts=coalesce(@SelValAnts + '' + ''''' + @separador + ''''' + '','''', '''') + ''convert(varchar(max), coalesce(d.'' + c.COLUMN_NAME + '',''''''''))'',
	@SelValors=coalesce(@SelValors + '' + ''''' + @separador + ''''' + '','''', '''') + ''convert(varchar(max), coalesce(i.'' + c.COLUMN_NAME + '',''''''''))''
FROM (SELECT ORDINAL_POSITION, COLUMN_NAME
	FROM INFORMATION_SCHEMA.COLUMNS
	WHERE TABLE_NAME=''' + @Taula + '''
	GROUP BY ORDINAL_POSITION, COLUMN_NAME) c
ORDER BY c.ORDINAL_POSITION ASC

' + -- Obté el codi d''usuari i el moment de l''operació:
'
SELECT @Usuari=system_user, @Moment=convert(varchar(23), getdate(), 109)

' + -- Obté el tipus d''operació que s''està realitzant:
'
IF exists (SELECT * FROM inserted)
	IF exists (SELECT * FROM deleted)
		SELECT @Tipus=''U''
	ELSE
		SELECT @Tipus=''I''
ELSE
	SELECT @Tipus=''D''

' + -- Es crea la query de la inserció de dades a la taula Auditories:
'
SELECT @sql=''insert Auditories (Tipus, Taula, PKCamps, PKValors, Camps, Valors, Operacio, Moment, Usuari)'' +
	'' SELECT '''''' + @Tipus + '''''''' +
	'',''''' + @Taula + ''''''' +
	'','' + @SelPKCamps +
	'','' + @SelPKValors +
	'','' + @SelNCamps

IF @Tipus=''D''
	SELECT @sql = @sql + '','''''' + @SelValAnts
ELSE
	SELECT @sql = @sql + '','''''' + @SelValors

SELECT @sql = @sql +
	'',''''TR_INS_' + @Taula +
	''''','''' + @Moment +
	'''','''' + @Usuari +
	'''' FROM #ins i full outer join #del d '' + @PKWhere

IF @Tipus=''U''
BEGIN

SELECT
	@SelNCampsMod=coalesce(@SelNCampsMod + '' + ''''' + @separador + ''''' + '','''', '''') + '''''''' + c.COLUMN_NAME + '''''''',
	@SelValAntsMod=coalesce(@SelValAntsMod + '' + ''''' + @separador + ''''' + '','''', '''') + ''convert(varchar(max), coalesce(d.'' + c.COLUMN_NAME + '',''''''''))'',
	@SelValorsMod=coalesce(@SelValorsMod + '' + ''''' + @separador + ''''' + '','''', '''') + ''convert(varchar(max), coalesce(i.'' + c.COLUMN_NAME + '',''''''''))''
FROM INFORMATION_SCHEMA.COLUMNS c
WHERE TABLE_NAME=''' + @Taula + ''' AND
sys.fn_IsBitSetInBitmask(COLUMNS_UPDATED(),
COLUMNPROPERTY(OBJECT_ID(TABLE_SCHEMA + ''.'' + TABLE_NAME), COLUMN_NAME, ''ColumnID'')) <> 0

END

EXEC (@sql)
'
SELECT len(@sql), @sql
EXEC(@sql)

	-- Itera a la taula següent del cursor:
    FETCH NEXT FROM table_cursor INTO @Taula
	END

-- Tanca i allibera el cursor de les taules:
CLOSE table_cursor
DEALLOCATE table_cursor
