﻿using fgxavi.Common.Services;
using fgxavi.Gupcat.Data.Entities;
using Microsoft.Data.Edm;
using Microsoft.Owin.Security.OAuth;
using System.Collections.Generic;
using System.Web.Http;
using System.Web.Http.OData.Builder;
using System.Web.Http.OData.Routing.Conventions;

namespace fgxavi.Gupcat.Presentation.Web
{
    public static class WebApiConfig
    {
        public static void Register(HttpConfiguration config)
        {
            // Configuración y servicios de Web API
            // Configure Web API para usar solo la autenticación de token de portador.
            config.SuppressDefaultHostAuthentication();
            config.Filters.Add(new HostAuthenticationFilter(OAuthDefaults.AuthenticationType));

            // Construeix les entitats del context per utilitzar-los als ODataController i oferir-los com a metadades.
            ODataConventionModelBuilder builder = new ODataConventionModelBuilder();
            //modelBuilder.Entity<Course>()
            //    .HasRequired(t => t.Department)
            //    .WithMany(t => t.Courses)
            //    .HasForeignKey(d => d.DepartmentID)
            //    .WillCascadeOnDelete(false);
            builder.EntitySet<Identitat>("Identitats");
            builder.EntitySet<TIP>("TIPs");
            builder.EntitySet<Usuari>("Usuaris");
            // Afegeix la opció de contar el nombre de resultats d'una consulta qualsevol.
            IList<IODataRoutingConvention> routingConventions = ODataRoutingConventions.CreateDefault();
            routingConventions.Insert(0, new CountODataRoutingConvention());
            routingConventions.Insert(0, new CountODataRoutingConvention());
            // Crea una acció nova que permet generar aleatòriament la base de dades de proves.
            ActionConfiguration generateSampleDb = builder.Entity<Usuari>().Collection.Action("GenerateSampleDatabase");
            generateSampleDb.Parameter<int>("usuaris");
            generateSampleDb.Returns<int>();
            // Defineix el mapa de rutes per consumir el servei OData.
            IEdmModel model = builder.GetEdmModel();
            config.Routes.MapODataRoute("odata", "odata", model, new CountODataPathHandler(), routingConventions);

            // Habilita l'execució de consultes per a tots els mètodes IQueryable dels control·ladors.
            config.EnableQuerySupport();

            // Rutas de Web API
            config.MapHttpAttributeRoutes();

            config.Routes.MapHttpRoute(
                name: "DefaultApi",
                routeTemplate: "api/{controller}/{id}",
                defaults: new { id = RouteParameter.Optional }
            );
        }
    }
}
