﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using System.Web.Http;
using System.Web.Http.ModelBinding;
using System.Web.Http.OData;
using System.Web.Http.OData.Routing;
using fgxavi.Gupcat.Data.Entities;

namespace fgxavi.Gupcat.Services.WebApiOData.Controllers
{
    /*
    Para agregar una ruta para este controlador, combine estas instrucciones al método Register de la clase WebApiConfig. Tenga en cuenta que las direcciones URL de OData distinguen mayúsculas de minúsculas.

    using System.Web.Http.OData.Builder;
    using fgxavi.Gupcat.Data.Entities;
    ODataConventionModelBuilder builder = new ODataConventionModelBuilder();
    builder.EntitySet<Identitat>("Identitat");
    builder.EntitySet<TIP>("TIP"); 
    config.Routes.MapODataRoute("odata", "odata", builder.GetEdmModel());
    */
    public class IdentitatController : ODataController
    {
        private GupcatEntities db = new GupcatEntities();

        // GET odata/Identitat
        //[Queryable]
        public IQueryable<Identitat> GetIdentitat()
        {
            //var result = new PageResult<Identitat>()
            return db.Identitats;
        }

        // GET odata/Identitat(5)
        [Queryable]
        public SingleResult<Identitat> GetIdentitat([FromODataUri] int key)
        {
            return SingleResult.Create(db.Identitats.Where(identitat => identitat.IdentitatId == key));
        }

        // PUT odata/Identitat(5)
        public async Task<IHttpActionResult> Put([FromODataUri] int key, Identitat identitat)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            if (key != identitat.IdentitatId)
            {
                return BadRequest();
            }

            db.Entry(identitat).State = EntityState.Modified;

            try
            {
                await db.SaveChangesAsync();
            }
            catch (DbUpdateConcurrencyException)
            {
                if (!IdentitatExists(key))
                {
                    return NotFound();
                }
                else
                {
                    throw;
                }
            }

            return Updated(identitat);
        }

        // POST odata/Identitat
        public async Task<IHttpActionResult> Post(Identitat identitat)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            db.Identitats.Add(identitat);
            await db.SaveChangesAsync();

            return Created(identitat);
        }

        // PATCH odata/Identitat(5)
        [AcceptVerbs("PATCH", "MERGE")]
        public async Task<IHttpActionResult> Patch([FromODataUri] int key, Delta<Identitat> patch)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            Identitat identitat = await db.Identitats.FindAsync(key);
            if (identitat == null)
            {
                return NotFound();
            }

            patch.Patch(identitat);

            try
            {
                await db.SaveChangesAsync();
            }
            catch (DbUpdateConcurrencyException)
            {
                if (!IdentitatExists(key))
                {
                    return NotFound();
                }
                else
                {
                    throw;
                }
            }

            return Updated(identitat);
        }

        // DELETE odata/Identitat(5)
        public async Task<IHttpActionResult> Delete([FromODataUri] int key)
        {
            Identitat identitat = await db.Identitats.FindAsync(key);
            if (identitat == null)
            {
                return NotFound();
            }

            db.Identitats.Remove(identitat);
            await db.SaveChangesAsync();

            return StatusCode(HttpStatusCode.NoContent);
        }

        // GET odata/Identitat(5)/TIPs
        [Queryable]
        public IQueryable<TIP> GetTIPs([FromODataUri] int key)
        {
            return db.Identitats.Where(m => m.IdentitatId == key).SelectMany(m => m.TIPs);
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                db.Dispose();
            }
            base.Dispose(disposing);
        }

        private bool IdentitatExists(int key)
        {
            return db.Identitats.Count(e => e.IdentitatId == key) > 0;
        }
    }
}
