﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using System.Web.Http;
using System.Web.Http.ModelBinding;
using System.Web.Http.OData;
using System.Web.Http.OData.Routing;
using fgxavi.Gupcat.Data.Entities;

namespace fgxavi.Gupcat.Services.WebApiOData.Controllers
{
    /*
    Para agregar una ruta para este controlador, combine estas instrucciones al método Register de la clase WebApiConfig. Tenga en cuenta que las direcciones URL de OData distinguen mayúsculas de minúsculas.

    using System.Web.Http.OData.Builder;
    using fgxavi.Gupcat.Data.Entities;
    ODataConventionModelBuilder builder = new ODataConventionModelBuilder();
    builder.EntitySet<TIP>("TIP");
    builder.EntitySet<Identitat>("Identitats"); 
    config.Routes.MapODataRoute("odata", "odata", builder.GetEdmModel());
    */
    public class TIPController : ODataController
    {
        private GupcatEntities db = new GupcatEntities();

        // GET odata/TIP
        [Queryable]
        public IQueryable<TIP> GetTIP()
        {
            return db.TIPs;
        }

        // GET odata/TIP(5)
        [Queryable]
        public SingleResult<TIP> GetTIP([FromODataUri] int key)
        {
            return SingleResult.Create(db.TIPs.Where(tip => tip.TIPId == key));
        }

        // PUT odata/TIP(5)
        public async Task<IHttpActionResult> Put([FromODataUri] int key, TIP tip)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            if (key != tip.TIPId)
            {
                return BadRequest();
            }

            db.Entry(tip).State = EntityState.Modified;

            try
            {
                await db.SaveChangesAsync();
            }
            catch (DbUpdateConcurrencyException)
            {
                if (!TIPExists(key))
                {
                    return NotFound();
                }
                else
                {
                    throw;
                }
            }

            return Updated(tip);
        }

        // POST odata/TIP
        public async Task<IHttpActionResult> Post(TIP tip)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            db.TIPs.Add(tip);
            await db.SaveChangesAsync();

            return Created(tip);
        }

        // PATCH odata/TIP(5)
        [AcceptVerbs("PATCH", "MERGE")]
        public async Task<IHttpActionResult> Patch([FromODataUri] int key, Delta<TIP> patch)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            TIP tip = await db.TIPs.FindAsync(key);
            if (tip == null)
            {
                return NotFound();
            }

            patch.Patch(tip);

            try
            {
                await db.SaveChangesAsync();
            }
            catch (DbUpdateConcurrencyException)
            {
                if (!TIPExists(key))
                {
                    return NotFound();
                }
                else
                {
                    throw;
                }
            }

            return Updated(tip);
        }

        // DELETE odata/TIP(5)
        public async Task<IHttpActionResult> Delete([FromODataUri] int key)
        {
            TIP tip = await db.TIPs.FindAsync(key);
            if (tip == null)
            {
                return NotFound();
            }

            db.TIPs.Remove(tip);
            await db.SaveChangesAsync();

            return StatusCode(HttpStatusCode.NoContent);
        }

        // GET odata/TIP(5)/Identitat
        [Queryable]
        public SingleResult<Identitat> GetIdentitat([FromODataUri] int key)
        {
            return SingleResult.Create(db.TIPs.Where(m => m.TIPId == key).Select(m => m.Identitat));
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                db.Dispose();
            }
            base.Dispose(disposing);
        }

        private bool TIPExists(int key)
        {
            return db.TIPs.Count(e => e.TIPId == key) > 0;
        }
    }
}
