﻿
namespace fgxavi.ListView
{
    public interface IListViewProvider<TSource, TQuery, TQueryKey, TQueryValue, TConnection, TElement>
    {
        /// <summary>
        /// Font base d'on s'obtenen els elements de l'enumeració.
        /// </summary>
        TSource Source { get; }
        /// <summary>
        /// Connexió preparada per establir comunicació amb la font de dades.
        /// </summary>
        TConnection Connection { get; }
        /// <summary>
        /// Factoria que es comunica amb la font de dades.
        /// </summary>
        IListViewFactory<TSource, TQuery, TQueryKey, TQueryValue, TConnection, TElement> Factory { get; }
        /// <summary>
        /// ViewModel amb les dades obtingudes de la font de dades.
        /// </summary>
        IListViewModel<TElement> ViewModel { get; }
    }
    public class ListViewProvider<TSource, TQuery, TQueryKey, TQueryValue, TConnection, TElement> : IListViewProvider<TSource, TQuery, TQueryKey, TQueryValue, TConnection, TElement>
    {
        public ListViewProvider(TSource Source,
            IListViewFactory<TSource, TQuery, TQueryKey, TQueryValue, TConnection, TElement> Factory,
            int[] PageSizeOptions = null,
            int MaxRange = ListViewModel<TElement>.DefaultMaxRange)
        {
            this.Source = Source;
            this.Factory = Factory;
            this.ViewModel = new ListViewModel<TElement>()
            {
                PageSizeOptions = PageSizeOptions ?? ListViewModel<TElement>.DefaultPageSizeOptions,
                MaxRange = MaxRange
            };
        }
        /// <summary>
        /// Font base d'on s'obtenen els elements de l'enumeració.
        /// </summary>
        public TSource Source { get; private set; }
        /// <summary>
        /// Connexió preparada per establir comunicació amb la font de dades.
        /// </summary>
        public TConnection Connection { get; private set; }
        /// <summary>
        /// Factoria que es comunica amb la font de dades.
        /// </summary>
        public IListViewFactory<TSource, TQuery, TQueryKey, TQueryValue, TConnection, TElement> Factory { get; private set; }
        /// <summary>
        /// ViewModel amb les dades obtingudes de la font de dades.
        /// </summary>
        public IListViewModel<TElement> ViewModel { get; private set; }
        /// <summary>
        /// Actualitza els elements de la llista mitjançant la font de dades.
        /// </summary>
        public virtual void Refresh()
        {
            this.Connection = Factory.ConnectionBuilder(this.Source, Factory.QueryOptions);
            Factory.Connector(this.Connection, this.ViewModel);
        }
    }
}
