﻿using System.Collections.Generic;

namespace fgxavi.ListView
{
    public interface IPagedQueryOption
    {
        int TotalItemCount { get; set; }
        int Top { get; }
        int Skip { get; }
    }
    public interface IPagedListView
    {
        int TotalItemCount { get; set; }
        int PageSize { get; set; }
        int PageNumber { get; set; }
        int PageCount { get; }
    }
    public interface IListViewPaginator
    {
        /// <summary>
        /// Opcions amb el nombre d'elements per pàgina.
        /// </summary>
        int[] PageSizeOptions { get; set; }
        /// <summary>
        /// Indicador de pàgines posteriors (automàtic).
        /// </summary>
        bool HasNextPage { get; }
        /// <summary>
        /// Indicador de pàgines anteriors (automàtic).
        /// </summary>
        bool HasPreviousPage { get; }
        /// <summary>
        /// Indicador de primera pàgina (automàtic).
        /// </summary>
        bool IsFirstPage { get; }
        /// <summary>
        /// Indicador d'última pàgina (automàtic).
        /// </summary>
        bool IsLastPage { get; }
        /// <summary>
        /// Indicador de segona pàgina (automàtic).
        /// </summary>
        bool IsSecondPage { get; }
        /// <summary>
        /// Indicador de penúltima pàgina (automàtic).
        /// </summary>
        bool IsPenultimatePage { get; }
        /// <summary>
        /// Índex del primer element de la pàgina actual (automàtic).
        /// </summary>
        int? FirstItem { get; }
        /// <summary>
        /// Índex de l'últim element de la pàgina actual (automàtic).
        /// </summary>
        int? LastItem { get; }
    }
    public interface IListViewNavigator
    {
        /// <summary>
        /// Nombre màxim de pàgines del rang seleccionable on pertany la pàgina actual.
        /// </summary>
        int MaxRange { get; set; }
        /// <summary>
        /// Array d'enters amb l'interval de pàgines del navegador.
        /// </summary>
        int[] Range { get; }
        /// <summary>
        /// Pàgina de navegació a l'esquerra o nul si no n'hi ha.
        /// </summary>
        int? LeftNavigator { get; }
        /// <summary>
        /// Pàgina de navegació a la dreta o nul si no n'hi ha.
        /// </summary>
        int? RightNavigator { get; }
    }
}
