﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using PagedList;
using System;
using System.Collections;
using System.Linq;

namespace fgxavi.PagedQuery
{
    public class PagedQueryConverter : JsonConverter
    {
        public const string PagedListMetadata = "PagedListMetadata";
        public const string PagedListItems = "Items";
        private int pageNumber;
        private int pageSize;
        public PagedQueryConverter()
            : this(PagedQueryExtendedExtensions.PageNumber, PagedQueryExtendedExtensions.PageSize)
        {
        }
        public PagedQueryConverter(int pageNumber, int pageSize)
        {
            this.pageNumber = pageNumber;
            this.pageSize = pageSize;
        }
        public override bool CanConvert(Type objectType)
        {
            return objectType.GetInterface(typeof(IEnumerable).FullName) != null
                && objectType.IsGenericType
                && !objectType.GenericTypeArguments.All(p => p.IsPrimitive);
        }
        public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
        {
            object obj = Activator.CreateInstance(objectType,
                new object[] { null, pageNumber, pageSize });
            JToken token = JToken.ReadFrom(reader);
            JsonConvert.PopulateObject(token[PagedListMetadata].ToString(), obj);

            //JToken token = JObject.Load(reader);
            //token.AddAfterSelf(JArray.Load(reader));
            //JObject jobj = JObject.Load(reader);
            string Console = token.ToString();
            while (reader.Read())
            {
                if (reader.TokenType == JsonToken.PropertyName && PagedListMetadata.Equals(reader.Value))
                {
                }
                if (reader.TokenType == JsonToken.PropertyName && PagedListItems.Equals(reader.Value))
                {
                }
                if (reader.Value != null)
                    Console += string.Format("Token: {0}, Value: {1}\n", reader.TokenType, reader.Value);
                else
                    Console += string.Format("Token: {0}\n", reader.TokenType);
            }
            return existingValue;
        }
        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {
            PagedListMetaData metadata;
            IEnumerable list;
            if (value is IPagedList && value is IEnumerable)
            {
                metadata = new PagedListMetaData(value as IPagedList);
                list = value as IEnumerable;
            }
            else if (value is IQueryable && value.GetType().IsGenericType)
            {
                Type generic = typeof(PagedList<>);
                Type type = generic.MakeGenericType(value.GetType().GenericTypeArguments);
                IQueryable query = value as IQueryable;
                object obj = Activator.CreateInstance(type,
                    new object[] { query, pageNumber, pageSize });
                metadata = new PagedListMetaData(obj as IPagedList);
                list = obj as IEnumerable;
            }
            else if (value is IEnumerable && value.GetType().IsGenericType)
            {
                Type generic = typeof(PagedList<>);
                Type type = generic.MakeGenericType(value.GetType().GenericTypeArguments);
                IEnumerable enumeration = value as IEnumerable;
                object obj = Activator.CreateInstance(type,
                    new object[] { enumeration, pageNumber, pageSize });
                metadata = new PagedListMetaData(obj as IPagedList);
                list = obj as IEnumerable;
            }
            else
            {
                writer.WriteNull();
                return;
            }
            writer.WriteStartObject();
            writer.WritePropertyName(PagedListMetadata);
            serializer.Serialize(writer, metadata);
            writer.WritePropertyName(PagedListItems);
            writer.WriteStartArray();
            foreach (var item in list)
                serializer.Serialize(writer, item);
            writer.WriteEndArray();
            writer.WriteEndObject();
        }
    }
}
