﻿using PagedList;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace fgxavi.PagedQuery
{
    public static class PagedQueryExtendedExtensions
    {
        #region Static members

        public static int PageNumber = 1;
        public static int PageSize = 10;

        #endregion

        #region Extensions

        public static IPagedQuery<T> ToPagedQuery<T>(this IQueryable<T> source) where T : class
        {
            return new PagedQuery<T>(source.OrderBy(p => true), PageNumber, PageSize);
        }
        public static IPagedQuery<T> ToPagedQuery<T>(this IQueryable<T> source, int pageNumber, int pageSize) where T : class
        {
            return new PagedQuery<T>(source.OrderBy(p => true), pageNumber, pageSize);
        }
        public static IPagedQuery<T> ToPagedQuery<T>(this IEnumerable<T> source) where T : class
        {
            return new PagedQuery<T>(source, PageNumber, PageSize);
        }
        public static IPagedQuery<T> ToPagedQuery<T>(this IEnumerable<T> source, int pageNumber, int pageSize) where T : class
        {
            return new PagedQuery<T>(source, pageNumber, pageSize);
        }
        public static IPagedQuery<T> AsPagedQuery<T>(this IPagedQuery<T> source) where T : class
        {
            return new PagedQuery<T>(source.OrderBy(p => true), source.PageNumber, source.PageSize);
        }

        #endregion

        #region Async Extensions

        public static Task<IPagedQuery<T>> ToPagedQueryAsync<T>(this IQueryable<T> source) where T : class
        {
            return Task.Run(() => source.ToPagedQuery());
        }
        public static Task<IPagedQuery<T>> ToPagedQueryAsync<T>(this IQueryable<T> source, int pageNumber, int pageSize) where T : class
        {
            return Task.Run(() => source.ToPagedQuery(pageNumber, pageSize));
        }
        public static Task<IPagedQuery<T>> ToPagedQueryAsync<T>(this IEnumerable<T> source) where T : class
        {
            return Task.Run(() => source.ToPagedQuery());
        }
        public static Task<IPagedQuery<T>> ToPagedQueryAsync<T>(this IEnumerable<T> source, int pageNumber, int pageSize) where T : class
        {
            return Task.Run(() => source.ToPagedQuery(pageNumber, pageSize));
        }
        public static Task<IPagedQuery<T>> ToPagedQueryAsync<T>(this IPagedQuery<T> source) where T : class
        {
            return Task.Run(() => source.ToPagedQuery());
        }

        #endregion
    }
}
